@@ -16,6 +16,21 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:element-hdv1394src
+ *
+ * Read MPEG-TS data from firewire port.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch hdv1394src ! queue ! decodebin name=d ! queue ! xvimagesink d. ! queue ! alsasink
+ * ]| captures from the firewire port and plays the streams.
+ * |[
+ * gst-launch hdv1394src ! queue ! filesink location=mydump.ts
+ * ]| capture to a disk file
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,6 +41,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <libavc1394/avc1394.h>
 #include <libavc1394/avc1394_vcr.h>
@@ -74,12 +90,6 @@ enum
   PROP_DEVICE_NAME
 };
 
-static const GstElementDetails gst_hdv1394src_details =
-GST_ELEMENT_DETAILS ("Firewire (1394) HDV video source",
-    "Source/Video",
-    "Source for MPEG-TS video data from firewire port",
-    "Edward Hervey <bilboed@bilboed.com>");
-
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -133,7 +143,10 @@ gst_hdv1394src_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_factory));
 
-  gst_element_class_set_details (element_class, &gst_hdv1394src_details);
+  gst_element_class_set_details_simple (element_class,
+      "Firewire (1394) HDV video source", "Source/Video",
+      "Source for MPEG-TS video data from firewire port",
+      "Edward Hervey <bilboed@bilboed.com>");
 }
 
 static void
@@ -153,18 +166,18 @@ gst_hdv1394src_class_init (GstHDV1394SrcClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
       g_param_spec_int ("port", "Port", "Port number (-1 automatic)",
-          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE));
+          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CHANNEL,
       g_param_spec_int ("channel", "Channel", "Channel number for listening",
-          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE));
+          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_USE_AVC,
       g_param_spec_boolean ("use-avc", "Use AV/C", "Use AV/C VTR control",
-          DEFAULT_USE_AVC, G_PARAM_READWRITE));
+          DEFAULT_USE_AVC, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_GUID,
       g_param_spec_uint64 ("guid", "GUID",
           "select one of multiple DV devices by its GUID. use a hexadecimal "
           "like 0xhhhhhhhhhhhhhhhh. (0 = no guid)", 0, G_MAXUINT64,
-          DEFAULT_GUID, G_PARAM_READWRITE));
+          DEFAULT_GUID, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstHDV1394Src:device-name
    *
@@ -174,7 +187,8 @@ gst_hdv1394src_class_init (GstHDV1394SrcClass * klass)
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "device name",
-          "user-friendly name of the device", "Default", G_PARAM_READABLE));
+          "user-friendly name of the device", "Default",
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gstbasesrc_class->negotiate = NULL;
   gstbasesrc_class->start = gst_hdv1394src_start;
@@ -298,11 +312,11 @@ gst_hdv1394src_iec61883_receive (unsigned char *data, int len,
   GST_LOG ("data:%p, len:%d, dropped:%d", data, len, dropped);
 
   /* error out if we don't have enough room ! */
-  if (dv1394src->outoffset > (2048 * 188 - len))
+  if (G_UNLIKELY (dv1394src->outoffset > (2048 * 188 - len)))
     return -1;
 
-  if (len == IEC61883_MPEG2_TSP_SIZE) {
-    memcpy (dv1394src->outdata + dv1394src->outoffset, data, len);
+  if (G_LIKELY (len == IEC61883_MPEG2_TSP_SIZE)) {
+    memcpy ((guint8 *) dv1394src->outdata + dv1394src->outoffset, data, len);
     dv1394src->outoffset += len;
   }
   dv1394src->frame_sequence++;
@@ -392,12 +406,13 @@ gst_hdv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
 
     GST_LOG ("res:%d", res);
 
-    if (res < 0) {
+    if (G_UNLIKELY (res < 0)) {
       if (errno == EAGAIN || errno == EINTR)
         continue;
       else
         goto error_while_polling;
     }
+
     if (G_UNLIKELY (pollfds[1].revents)) {
       char command;
 
@@ -421,7 +436,7 @@ gst_hdv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
 
   g_assert (dv1394src->outoffset);
 
-  GST_LOG ("We have some frames (%d bytes)", dv1394src->outoffset);
+  GST_LOG ("We have some frames (%u bytes)", (guint) dv1394src->outoffset);
 
   /* Create the buffer */
   *buf = gst_buffer_new ();
@@ -465,7 +480,6 @@ gst_hdv1394src_discover_avc_node (GstHDV1394Src * src)
   /* loop over all our ports */
   for (; j < m && node == -1; j++) {
     raw1394handle_t handle;
-    gint n_ports;
     struct raw1394_portinfo pinf[16];
 
     /* open the port */
@@ -474,7 +488,7 @@ gst_hdv1394src_discover_avc_node (GstHDV1394Src * src)
       GST_WARNING ("raw1394 - failed to get handle: %s.\n", strerror (errno));
       continue;
     }
-    if ((n_ports = raw1394_get_port_info (handle, pinf, 16)) < 0) {
+    if (raw1394_get_port_info (handle, pinf, 16) < 0) {
       GST_WARNING ("raw1394 - failed to get port info: %s.\n",
           strerror (errno));
       goto next;
@@ -776,13 +790,15 @@ gst_hdv1394src_uri_get_type (void)
 {
   return GST_URI_SRC;
 }
+
 static gchar **
 gst_hdv1394src_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "hdv", NULL };
+  static gchar *protocols[] = { (char *) "hdv", NULL };
 
   return protocols;
 }
+
 static const gchar *
 gst_hdv1394src_uri_get_uri (GstURIHandler * handler)
 {
