@@ -42,13 +42,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_multipart_mux_debug);
 #define GST_CAT_DEFAULT gst_multipart_mux_debug
 
-/* elementfactory information */
-static const GstElementDetails gst_multipart_mux_details =
-GST_ELEMENT_DETAILS ("Multipart muxer",
-    "Codec/Muxer",
-    "mux multipart streams",
-    "Wim Taymans <wim@fluendo.com>");
-
 #define DEFAULT_BOUNDARY        "ThisRandomString"
 
 enum
@@ -140,7 +133,8 @@ gst_multipart_mux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory));
 
-  gst_element_class_set_details (element_class, &gst_multipart_mux_details);
+  gst_element_class_set_details_simple (element_class, "Multipart muxer",
+      "Codec/Muxer", "mux multipart streams", "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -161,7 +155,7 @@ gst_multipart_mux_class_init (GstMultipartMuxClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BOUNDARY,
       g_param_spec_string ("boundary", "Boundary", "Boundary string",
-          DEFAULT_BOUNDARY, G_PARAM_READWRITE));
+          DEFAULT_BOUNDARY, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->request_new_pad = gst_multipart_mux_request_new_pad;
   gstelement_class->change_state = gst_multipart_mux_change_state;
@@ -231,11 +225,11 @@ gst_multipart_mux_request_new_pad (GstElement * element,
   /* construct our own wrapper data structure for the pad to
    * keep track of its status */
   {
-    GstMultipartPad *multipartpad;
+    GstMultipartPadData *multipartpad;
 
-    multipartpad = (GstMultipartPad *)
+    multipartpad = (GstMultipartPadData *)
         gst_collect_pads_add_pad (multipart_mux->collect, newpad,
-        sizeof (GstMultipartPad));
+        sizeof (GstMultipartPadData));
 
     /* save a pointer to our data in the pad */
     gst_pad_set_element_private (newpad, multipartpad);
@@ -338,7 +332,7 @@ gst_multipart_mux_get_mime (GstMultipartMux * mux, GstStructure * s)
  */
 static gint
 gst_multipart_mux_compare_pads (GstMultipartMux * multipart_mux,
-    GstMultipartPad * old, GstMultipartPad * new)
+    GstMultipartPadData * old, GstMultipartPadData * new)
 {
   guint64 oldtime, newtime;
 
@@ -372,11 +366,11 @@ gst_multipart_mux_compare_pads (GstMultipartMux * multipart_mux,
 
 /* make sure a buffer is queued on all pads, returns a pointer to an multipartpad
  * that holds the best buffer or NULL when no pad was usable */
-static GstMultipartPad *
+static GstMultipartPadData *
 gst_multipart_mux_queue_pads (GstMultipartMux * mux)
 {
   GSList *walk = NULL;
-  GstMultipartPad *bestpad = NULL;
+  GstMultipartPadData *bestpad = NULL;
 
   g_return_val_if_fail (GST_IS_MULTIPART_MUX (mux), NULL);
 
@@ -384,7 +378,7 @@ gst_multipart_mux_queue_pads (GstMultipartMux * mux)
   walk = mux->collect->data;
   while (walk) {
     GstCollectData *data = (GstCollectData *) walk->data;
-    GstMultipartPad *pad = (GstMultipartPad *) data;
+    GstMultipartPadData *pad = (GstMultipartPadData *) data;
 
     walk = g_slist_next (walk);
 
@@ -428,7 +422,7 @@ gst_multipart_mux_queue_pads (GstMultipartMux * mux)
 static GstFlowReturn
 gst_multipart_mux_collected (GstCollectPads * pads, GstMultipartMux * mux)
 {
-  GstMultipartPad *best;
+  GstMultipartPadData *best;
   GstFlowReturn ret = GST_FLOW_OK;
   gchar *header = NULL;
   size_t headerlen;
