@@ -37,13 +37,6 @@ GST_DEBUG_CATEGORY_STATIC (rtpamrdepay_debug);
  * Wideband (AMR-WB) Audio Codecs.
  */
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_amrdepay_details =
-GST_ELEMENT_DETAILS ("RTP AMR depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts AMR or AMR-WB audio from RTP packets (RFC 3267)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 /* RtpAMRDepay signals and args */
 enum
 {
@@ -131,22 +124,19 @@ gst_rtp_amr_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_amr_depay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_amrdepay_details);
+  gst_element_class_set_details_simple (element_class, "RTP AMR depayloader",
+      "Codec/Depayloader/Network/RTP",
+      "Extracts AMR or AMR-WB audio from RTP packets (RFC 3267)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
 gst_rtp_amr_depay_class_init (GstRtpAMRDepayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gstbasertpdepayload_class->process = gst_rtp_amr_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_amr_depay_setcaps;
 
@@ -270,22 +260,23 @@ invalid_mode:
 }
 
 /* -1 is invalid */
-static gint nb_frame_size[16] = {
+static const gint nb_frame_size[16] = {
   12, 13, 15, 17, 19, 20, 26, 31,
   5, -1, -1, -1, -1, -1, -1, 0
 };
-static gint wb_frame_size[16] = {
+
+static const gint wb_frame_size[16] = {
   17, 23, 32, 36, 40, 46, 50, 58,
-  60, -1, -1, -1, -1, -1, -1, 0
+  60, 5, -1, -1, -1, -1, -1, 0
 };
 
 static GstBuffer *
 gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 {
   GstRtpAMRDepay *rtpamrdepay;
+  const gint *frame_size;
   GstBuffer *outbuf = NULL;
   gint payload_len;
-  gint *frame_size;
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
 
@@ -295,7 +286,7 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   else
     frame_size = wb_frame_size;
 
-  /* when we get here, 1 channel, 8000/16000 Hz, octet aligned, no CRC, 
+  /* when we get here, 1 channel, 8000/16000 Hz, octet aligned, no CRC,
    * no robust sorting, no interleaving data is to be depayloaded */
   {
     guint8 *payload, *p, *dp;
@@ -303,9 +294,7 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     gint i, num_packets, num_nonempty_packets;
     gint amr_len;
     gint ILL, ILP;
-    gboolean marker;
 
-    marker = gst_rtp_buffer_get_marker (buf);
     payload_len = gst_rtp_buffer_get_payload_len (buf);
 
     /* need at least 2 bytes for the header */
@@ -317,7 +306,7 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     /* depay CMR. The CMR is used by the sender to request
      * a new encoding mode.
      *
-     *  0 1 2 3 4 5 6 7 
+     *  0 1 2 3 4 5 6 7
      * +-+-+-+-+-+-+-+-+
      * | CMR   |R|R|R|R|
      * +-+-+-+-+-+-+-+-+
@@ -341,8 +330,8 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         goto wrong_interleaving;
     }
 
-    /* 
-     *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
+    /*
+     *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6
      * +-+-+-+-+-+-+-+-+..
      * |F|  FT   |Q|P|P| more FT..
      * +-+-+-+-+-+-+-+-+..
@@ -413,11 +402,10 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     /* we can set the duration because each packet is 20 milliseconds */
     GST_BUFFER_DURATION (outbuf) = num_packets * 20 * GST_MSECOND;
 
-    if (marker) {
+    if (gst_rtp_buffer_get_marker (buf)) {
       /* marker bit marks a discont buffer after a talkspurt. */
       GST_DEBUG_OBJECT (depayload, "marker bit was set");
       GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
-      marker = FALSE;
     }
 
     GST_DEBUG_OBJECT (depayload, "pushing buffer of size %d",
@@ -467,5 +455,5 @@ gboolean
 gst_rtp_amr_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpamrdepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_AMR_DEPAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_AMR_DEPAY);
 }
