@@ -1,5 +1,5 @@
-/* Gnome-Streamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+/* GStreamer
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,7 @@
 #define __GST_RTP_MPA_DEPAY_H__
 
 #include <gst/gst.h>
+#include <gst/rtp/gstbasertpdepayload.h>
 
 G_BEGIN_DECLS
 
@@ -29,10 +30,10 @@ G_BEGIN_DECLS
 #define GST_RTP_MPA_DEPAY(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_RTP_MPA_DEPAY,GstRtpMPADepay))
 #define GST_RTP_MPA_DEPAY_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_RTP_MPA_DEPAY,GstRtpMPADepay))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_RTP_MPA_DEPAY,GstRtpMPADepayClass))
 #define GST_IS_RTP_MPA_DEPAY(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_RTP_MPA_DEPAY))
-#define GST_IS_RTP_MPA_DEPAY_CLASS(obj) \
+#define GST_IS_RTP_MPA_DEPAY_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RTP_MPA_DEPAY))
 
 typedef struct _GstRtpMPADepay GstRtpMPADepay;
@@ -40,19 +41,16 @@ typedef struct _GstRtpMPADepayClass GstRtpMPADepayClass;
 
 struct _GstRtpMPADepay
 {
-  GstElement element;
-
-  GstPad *sinkpad;
-  GstPad *srcpad;
-
-  guint frequency;
+  GstBaseRTPDepayload depayload;
 };
 
 struct _GstRtpMPADepayClass
 {
-  GstElementClass parent_class;
+  GstBaseRTPDepayloadClass parent_class;
 };
 
+GType gst_rtp_mpa_depay_get_type (void);
+
 gboolean gst_rtp_mpa_depay_plugin_init (GstPlugin * plugin);
 
 G_END_DECLS
