@@ -283,7 +283,7 @@ update_buffer_level (RTPJitterBuffer * jbuf, gint * percent)
   if (post) {
     gint perc;
 
-    if (jbuf->buffering && (jbuf->high_level != 0)) {
+    if (jbuf->buffering) {
       perc = (level * 100 / jbuf->high_level);
       perc = MIN (perc, 100);
     } else {
@@ -634,26 +634,6 @@ rtp_jitter_buffer_insert (RTPJitterBuffer * jbuf, GstBuffer * buf,
   }
 
   rtptime = gst_rtp_buffer_get_timestamp (buf);
-  /* rtp time jumps are checked for during skew calculation, but bypassed
-   * in other mode, so mind those here and reset jb if needed.
-   * Only reset if valid input time, which is likely for UDP input
-   * where we expect this might happen due to async thread effects
-   * (in seek and state change cycles), but not so much for TCP input */
-  if (GST_CLOCK_TIME_IS_VALID (time) &&
-      jbuf->mode != RTP_JITTER_BUFFER_MODE_SLAVE &&
-      jbuf->base_time != -1 && jbuf->last_rtptime != -1) {
-    GstClockTime ext_rtptime = jbuf->ext_rtptime;
-
-    ext_rtptime = gst_rtp_buffer_ext_timestamp (&ext_rtptime, rtptime);
-    if (ext_rtptime > jbuf->last_rtptime + 3 * clock_rate ||
-        ext_rtptime + 3 * clock_rate < jbuf->last_rtptime) {
-      /* reset even if we don't have valid incoming time;
-       * still better than producing possibly very bogus output timestamp */
-      GST_WARNING ("rtp delta too big, reset skew");
-      rtp_jitter_buffer_reset_skew (jbuf);
-    }
-  }
-
   switch (jbuf->mode) {
     case RTP_JITTER_BUFFER_MODE_NONE:
     case RTP_JITTER_BUFFER_MODE_BUFFER:
@@ -719,7 +699,7 @@ rtp_jitter_buffer_pop (RTPJitterBuffer * jbuf, gint * percent)
 {
   GstBuffer *buf;
 
-  g_return_val_if_fail (jbuf != NULL, NULL);
+  g_return_val_if_fail (jbuf != NULL, FALSE);
 
   buf = g_queue_pop_tail (jbuf->packets);
 
@@ -747,7 +727,7 @@ rtp_jitter_buffer_peek (RTPJitterBuffer * jbuf)
 {
   GstBuffer *buf;
 
-  g_return_val_if_fail (jbuf != NULL, NULL);
+  g_return_val_if_fail (jbuf != NULL, FALSE);
 
   buf = g_queue_peek_tail (jbuf->packets);
 
