@@ -46,13 +46,6 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-/* elementfactory information */
-static const GstElementDetails gst_dynudpsink_details =
-GST_ELEMENT_DETAILS ("UDP packet sender",
-    "Sink/Network",
-    "Send data over the network via UDP",
-    "Philippe Khalaf <burger@speedy.org>");
-
 /* DynUDPSink signals and args */
 enum
 {
@@ -139,7 +132,10 @@ gst_dynudpsink_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
 
-  gst_element_class_set_details (element_class, &gst_dynudpsink_details);
+  gst_element_class_set_details_simple (element_class, "UDP packet sender",
+      "Sink/Network",
+      "Send data over the network via UDP",
+      "Philippe Khalaf <burger@speedy.org>");
 }
 
 static void
@@ -168,11 +164,12 @@ gst_dynudpsink_class_init (GstDynUDPSink * klass)
   g_object_class_install_property (gobject_class, PROP_SOCKFD,
       g_param_spec_int ("sockfd", "socket handle",
           "Socket to use for UDP sending. (-1 == allocate)",
-          -1, G_MAXINT16, UDP_DEFAULT_SOCKFD, G_PARAM_READWRITE));
+          -1, G_MAXINT16, UDP_DEFAULT_SOCKFD,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CLOSEFD,
       g_param_spec_boolean ("closefd", "Close sockfd",
           "Close sockfd if passed as property on state change",
-          UDP_DEFAULT_CLOSEFD, G_PARAM_READWRITE));
+          UDP_DEFAULT_CLOSEFD, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = gst_dynudpsink_change_state;
 
@@ -243,7 +240,11 @@ gst_dynudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   theiraddr.sin_family = AF_INET;
   theiraddr.sin_addr.s_addr = destaddr;
   theiraddr.sin_port = destport;
+#ifdef G_OS_WIN32
+  ret = sendto (sink->sock, (char *) data, size, 0,
+#else
   ret = sendto (sink->sock, data, size, 0,
+#endif
       (struct sockaddr *) &theiraddr, sizeof (theiraddr));
 
   if (ret < 0) {
@@ -316,7 +317,6 @@ static gboolean
 gst_dynudpsink_init_send (GstDynUDPSink * sink)
 {
   guint bc_val;
-  gint ret;
 
   if (sink->sockfd == -1) {
     /* create sender socket if none available */
@@ -324,9 +324,8 @@ gst_dynudpsink_init_send (GstDynUDPSink * sink)
       goto no_socket;
 
     bc_val = 1;
-    if ((ret =
-            setsockopt (sink->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
-                sizeof (bc_val))) < 0)
+    if (setsockopt (sink->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
+            sizeof (bc_val)) < 0)
       goto no_broadcast;
 
     sink->externalfd = TRUE;
