@@ -22,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
@@ -40,7 +41,7 @@ static GtkWidget *drawingarea = NULL;
 static GstClock *sync_clock = NULL;
 
 static void
-on_window_destroy (GtkObject * object, gpointer user_data)
+on_window_destroy (GObject * object, gpointer user_data)
 {
   drawingarea = NULL;
   gtk_main_quit ();
@@ -77,32 +78,44 @@ draw_spectrum (gfloat * data)
 {
   gint i;
   GdkRectangle rect = { 0, 0, spect_bands, spect_height };
+  cairo_t *cr;
 
   if (!drawingarea)
     return;
 
-  gdk_window_begin_paint_rect (drawingarea->window, &rect);
-  gdk_draw_rectangle (drawingarea->window, drawingarea->style->black_gc,
-      TRUE, 0, 0, spect_bands, spect_height);
+  gdk_window_begin_paint_rect (gtk_widget_get_window (drawingarea), &rect);
+  cr = gdk_cairo_create (gtk_widget_get_window (drawingarea));
+
+  cairo_set_source_rgb (cr, 0, 0, 0);
+  cairo_rectangle (cr, 0, 0, spect_bands, spect_height);
+  cairo_fill (cr);
+  cairo_set_source_rgb (cr, 1, 1, 1);
   for (i = 0; i < spect_bands; i++) {
-    gdk_draw_rectangle (drawingarea->window, drawingarea->style->white_gc,
-        TRUE, i, -data[i], 1, spect_height + data[i]);
+    cairo_rectangle (cr, i, -data[i], 1, spect_height + data[i]);
+    cairo_fill (cr);
   }
-  gdk_window_end_paint (drawingarea->window);
+  cairo_destroy (cr);
+
+  gdk_window_end_paint (gtk_widget_get_window (drawingarea));
 }
 
 /* process delayed message */
+static gboolean
+delayed_idle_spectrum_update (gpointer user_data)
+{
+  draw_spectrum ((gfloat *) user_data);
+  g_free (user_data);
+  return (FALSE);
+}
+
 static gboolean
 delayed_spectrum_update (GstClock * sync_clock, GstClockTime time,
     GstClockID id, gpointer user_data)
 {
-  if (!GST_CLOCK_TIME_IS_VALID (time))
-    goto done;
-
-  draw_spectrum ((gfloat *) user_data);
-
-done:
-  g_free (user_data);
+  if (GST_CLOCK_TIME_IS_VALID (time))
+    g_idle_add (delayed_idle_spectrum_update, user_data);
+  else
+    g_free (user_data);
   return (TRUE);
 }
 
